# CPL_CHECK_CFITSIO
#------------------
# Checks for the cfitsio library and header files.
AC_DEFUN([CPL_CHECK_CFITSIO],
[

    AC_REQUIRE([CPL_CHECK_QFITS])

    AC_MSG_CHECKING([for cfitsio])

    cpl_cfitsio_check_header="fitsio.h"
    cpl_cfitsio_check_lib="libcfitsio.a"

    cpl_cfitsio_includes=""
    cpl_cfitsio_libraries=""

    AC_ARG_WITH(cfitsio,
                AC_HELP_STRING([--with-cfitsio],
                               [location where cfitsio is installed]),
                [
                    cpl_with_cfitsio_includes=$withval/include
                    cpl_with_cfitsio_libs=$withval/lib
                ])

    AC_ARG_WITH(cfitsio-includes,
                AC_HELP_STRING([--with-cfitsio-includes],
                               [location of the cfitsio header files]),
                cpl_with_cfitsio_includes=$withval)

    AC_ARG_WITH(cfitsio-libs,
                AC_HELP_STRING([--with-cfitsio-libs],
                               [location of the cfitsio library]),
                cpl_with_cfitsio_libs=$withval)

    AC_ARG_ENABLE(cfitsio-test,
                  AC_HELP_STRING([--disable-cfitsio-test],
                                 [disables checks for the cfitsio library and headers]),
                  cpl_enable_cfitsio_test=$enableval,
                  cpl_enable_cfitsio_test=yes)


    if test "x$cpl_enable_cfitsio_test" = xyes; then

        # Check for the cfitsio includes

        if test -z "$cpl_with_cfitsio_includes"; then
            cpl_cfitsio_incdirs="/opt/cfitsio/include /usr/local/include /usr/include"

            test -n "$CPLDIR" && cpl_cfitsio_incdirs="$CPLDIR/include \
                $cpl_cfitsio_incdirs"

            test -n "$CFITSIODIR" && cpl_cfitsio_incdirs="$CFITSIODIR/include \
                $cpl_cfitsio_incdirs"
        else
            cpl_cfitsio_incdirs="$cpl_with_cfitsio_includes"
        fi

        ESO_FIND_FILE($cpl_cfitsio_check_header, $cpl_cfitsio_incdirs,
                      cpl_cfitsio_includes)


        # Check for the cfitsio library

        if test -z "$cpl_with_cfitsio_libs"; then
            cpl_cfitsio_libdirs="/opt/cfitsio/lib /usr/local/lib /usr/lib"

            test -n "$CPLDIR" && cpl_cfitsio_libdirs="$CPLDIR/lib \
                $cpl_cfitsio_libdirs"

            test -n "$CFITSIODIR" && cpl_cfitsio_libdirs="$CFITSIODIR/lib \
                $cpl_cfitsio_libdirs"
        else
            cpl_cfitsio_libdirs="$cpl_with_cfitsio_libs"
        fi

        ESO_FIND_FILE($cpl_cfitsio_check_lib, $cpl_cfitsio_libdirs,
                      cpl_cfitsio_libraries)


        if test x"$cpl_cfitsio_includes" = xno || \
            test x"$cpl_cfitsio_libraries" = xno; then
            cpl_cfitsio_notfound=""

            if test x"$cpl_cfitsio_includes" = xno; then
                if test x"$cpl_cfitsio_libraries" = xno; then
                    cpl_cfitsio_notfound="(headers and libraries)"
                else
                    cpl_cfitsio_notfound="(headers)"
                fi
            else
                cpl_cfitsio_notfound="(libraries)"
            fi

            AC_MSG_ERROR([cfitsio $cpl_cfitsio_notfound was not found on your system. Please check!])
        else
            AC_MSG_RESULT([libraries $cpl_cfitsio_libraries, headers $cpl_cfitsio_includes])
        fi

        # Set up the symbols

        CFITSIO_INCLUDES="-I$cpl_cfitsio_includes"
        CFITSIO_LDFLAGS="-L$cpl_cfitsio_libraries"
        LIBCFITSIO="-lcfitsio"
    else
        AC_MSG_RESULT([disabled])
        AC_MSG_WARN([cfitsio checks have been disabled! This package may not build!])
        CFITSIO_INCLUDES=""
        CFITSIO_LDFLAGS=""
        LIBCFITSIO=""
    fi

    AC_SUBST(CFITSIO_INCLUDES)
    AC_SUBST(CFITSIO_LDFLAGS)
    AC_SUBST(LIBCFITSIO)

])

# CPL_CHECK_QFITS
#----------------
# Checks for the Qfits library and header files.
AC_DEFUN([CPL_CHECK_QFITS],
[

    AC_MSG_CHECKING([for Qfits])

    cpl_qfits_check_header="qfits.h"
    cpl_qfits_check_lib="libqfits.a"

    cpl_qfits_includes=""
    cpl_qfits_libraries=""

    AC_ARG_WITH(qfits,
                AC_HELP_STRING([--with-qfits],
                               [location where Qfits is installed]),
                [
                    cpl_with_qfits_includes=$withval/include
                    cpl_with_qfits_libs=$withval/lib
                ])

    AC_ARG_WITH(qfits-includes,
                AC_HELP_STRING([--with-qfits-includes],
                               [location of the Qfits header files]),
                cpl_with_qfits_includes=$withval)

    AC_ARG_WITH(qfits-libs,
                AC_HELP_STRING([--with-qfits-libs],
                               [location of the Qfits library]),
                cpl_with_qfits_libs=$withval)

    AC_ARG_ENABLE(qfits-test,
                  AC_HELP_STRING([--disable-qfits-test],
                                 [disables checks for the Qfits library and headers]),
                  cpl_enable_qfits_test=$enableval,
                  cpl_enable_qfits_test=yes)


    if test "x$cpl_enable_qfits_test" = xyes; then

        # Check for the Qfits includes

        if test -z "$cpl_with_qfits_includes"; then
            cpl_qfits_incdirs="/opt/qfits/include /usr/local/include /usr/include"

            test -n "$CPLDIR" && cpl_qfits_incdirs="$CPLDIR/include \
                                                    $cpl_qfits_incdirs"

            test -n "$QFITSDIR" && cpl_qfits_incdirs="$QFITSDIR/include \
                                                      $cpl_qfits_incdirs"
        else
            cpl_qfits_incdirs="$cpl_with_qfits_includes"
        fi

        ESO_FIND_FILE($cpl_qfits_check_header, $cpl_qfits_incdirs,
                      cpl_qfits_includes)


        # Check for the Qfits library

        if test -z "$cpl_with_qfits_libs"; then
            cpl_qfits_libdirs="/opt/qfits/lib /usr/local/lib /usr/lib"

            test -n "$CPLDIR" && cpl_qfits_libdirs="$CPLDIR/lib \
                                                    $cpl_qfits_libdirs"

            test -n "$QFITSDIR" && cpl_qfits_libdirs="$QFITSDIR/lib \
                                                      $cpl_qfits_libdirs"
        else
            cpl_qfits_libdirs="$cpl_with_qfits_libs"
        fi

        ESO_FIND_FILE($cpl_qfits_check_lib, $cpl_qfits_libdirs,
                      cpl_qfits_libraries)


        if test x"$cpl_qfits_includes" = xno || \
            test x"$cpl_qfits_libraries" = xno; then
            cpl_qfits_notfound=""

            if test x"$cpl_qfits_includes" = xno; then
                if test x"$cpl_qfits_libraries" = xno; then
                    cpl_qfits_notfound="(headers and libraries)"
                else            
                    cpl_qfits_notfound="(headers)"
                fi
            else
                cpl_qfits_notfound="(libraries)"
            fi

            AC_MSG_ERROR([Qfits $cpl_qfits_notfound was not found on your system. Please check!])
        else
            AC_MSG_RESULT([libraries $cpl_qfits_libraries, headers $cpl_qfits_includes])
        fi


        # Set up the symbols

        QFITS_INCLUDES="-I$cpl_qfits_includes"
        QFITS_LDFLAGS="-L$cpl_qfits_libraries"
        LIBQFITS="-lqfits"
    else
        AC_MSG_RESULT([disabled])
        AC_MSG_WARN([Qfits checks have been disabled! This package may not build!])
        QFITS_INCLUDES=""
        QFITS_LDFLAGS=""
        LIBQFITS=""
    fi

    AC_SUBST(QFITS_INCLUDES)
    AC_SUBST(QFITS_LDFLAGS)
    AC_SUBST(LIBQFITS)

    # Check whether we are using Qfits 4.3.5 or a later version

    AC_CACHE_CHECK([whether Qfits provides qfits_get_datetime_iso8601],
                   [cpl_cv_func_qfits_get_datetime_iso8601],
                   [
                       AC_LANG_PUSH(C)

                       cpl_cppflags_save="$CPPFLAGS"
                       cpl_cflags_save="$CFLAGS"
                       cpl_ldflags_save="$LDFLAGS"
                       cpl_libs_save="$LIBS"

                       if test x$GCC = xyes; then
                           CFLAGS="$CFLAGS -pedantic-errors"
                           CPPFLAGS="$CPPFLAGS $CFLAGS"
                       fi

                       CFLAGS="$CFLAGS $QFITS_INCLUDES"
                       LDFLAGS="$LDFLAGS $QFITS_LDFLAGS"
                       LIBS="$LIBQFITS $LIBS"

                       AC_LINK_IFELSE([
#include <qfits.h>

int
main(void)
{

    const char *s = NULL;

    s = qfits_get_datetime_iso8601();
    return 0;

}
                                     ],
                                     cpl_cv_func_qfits_get_datetime_iso8601=yes,
                                     cpl_cv_func_qfits_get_datetime_iso8601=no,
                                     cpl_cv_func_qfits_get_datetime_iso8601=no)


                       CPPFLAGS="$cpl_cppflags_save"
                       CFLAGS="$cpl_cflags_save"
                       LDFLAGS="$cpl_ldflags_save"
                       LIBS="$cpl_libs_save"

                       AC_LANG_POP(C)

                   ])

    AH_TEMPLATE([HAVE_QFITS_GET_DATETIME_ISO8601],
                [Define if Qfits provides the `qfits_get_datetime_iso8601' 
                 function])

    if test x$cpl_cv_func_qfits_get_datetime_iso8601 = xyes; then
        AC_DEFINE(HAVE_QFITS_GET_DATETIME_ISO8601)
    fi

])




#
# CPL_CREATE_SYMBOLS(build=[])
#-----------------------------
# Sets the Makefile symbols for the CPL and C extension libraries. If an
# argument is provided the symbols are setup for building CPL, if no
# argument is given (default) the symbols are set for using the libraries
# for external package development.
AC_DEFUN([CPL_CREATE_SYMBOLS],
[
    if test -z "$1"; then
        LIBCPLCORE='-lcplcore'
        LIBCPLDRS='-lcpldrs'
        LIBCPLUI='-lcplui'
        LIBCPLDFS='-lcpldfs'
    else
        LIBCPLCORE='$(top_builddir)/cplcore/libcplcore.la'
        LIBCPLDRS='$(top_builddir)/cpldrs/libcpldrs.la'
        LIBCPLUI='$(top_builddir)/cplui/libcplui.la'
        LIBCPLDFS='$(top_builddir)/cpldfs/libcpldfs.la'
    fi
   AC_SUBST(LIBCPLCORE)
   AC_SUBST(LIBCPLDRS)
   AC_SUBST(LIBCPLUI)
   AC_SUBST(LIBCPLDFS)
])


# CPL_CHECK_LIBS
#---------------
# Checks for the CPL libraries and header files.
AC_DEFUN([CPL_CHECK_LIBS],
[

    AC_MSG_CHECKING([for CPL])

    cpl_check_cpl_header="cpl_macros.h"
    cpl_check_cpl_lib="libcplcore.la"

    cpl_includes=""
    cpl_libraries=""

    AC_ARG_WITH(cpl,
                AC_HELP_STRING([--with-cpl],
                               [location where CPL is installed]),
                [
                    cpl_with_cpl_includes=$withval/include
                    cpl_with_cpl_libs=$withval/lib
                ])

    AC_ARG_WITH(cpl-includes,
                AC_HELP_STRING([--with-cpl-includes],
                               [location of the CPL header files]),
                cpl_with_cpl_includes=$withval)

    AC_ARG_WITH(cpl-libs,
                AC_HELP_STRING([--with-cpl-libs],
                               [location of the CPL library]),
                cpl_with_cpl_libs=$withval)

    AC_ARG_ENABLE(cpl-test,
                  AC_HELP_STRING([--disable-cpl-test],
                                 [disables checks for the CPL library and headers]),
                  cpl_enable_cpl_test=$enableval,
                  cpl_enable_cpl_test=yes)


    if test "x$cpl_enable_cpl_test" = xyes; then

        # Check for the CPL includes

        if test -z "$cpl_with_cpl_includes"; then
            cpl_incdirs="/opt/cpl/include \
                         /usr/local/include \
                         /usr/local/cpl/include \
                         /usr/local/include/cpl \
                         /usr/include/cpl \
                         /usr/include"

            test -n "$CPLDIR" && cpl_incdirs="$CPLDIR/include $cpl_incdirs"
        else
            cpl_incdirs="$cpl_with_cpl_includes"
        fi

        ESO_FIND_FILE($cpl_check_cpl_header, $cpl_incdirs, cpl_includes)


        # Check for the CPL libraries

        if test -z "$cpl_with_cpl_libs"; then
            cpl_libdirs="/opt/cpl/lib \
                         /usr/local/lib \
                         /usr/local/cpl/lib \
                         /usr/lib"

            test -n "$CPLDIR" && cpl_libdirs="$CPLDIR/lib $cpl_libdirs"
        else
            cpl_libdirs="$cpl_with_cpl_libs"
        fi

        ESO_FIND_FILE($cpl_check_cpl_lib, $cpl_libdirs, cpl_libraries)


        if test x"$cpl_includes" = xno || test x"$cpl_libraries" = xno; then
            cpl_notfound=""

            if test x"$cpl_includes" = xno; then
                if test x"$cpl_libraries" = xno; then
                    cpl_notfound="(headers and libraries)"
                else            
                    cpl_notfound="(headers)"
                fi
            else
                cpl_notfound="(libraries)"
            fi

            AC_MSG_ERROR([CPL $cpl_notfound was not found on your system. Please check!])
        else
            AC_MSG_RESULT([libraries $cpl_libraries, headers $cpl_includes])
        fi

        # Set up the symbols

        CPL_INCLUDES="-I$cpl_includes"
        CPL_LDFLAGS="-L$cpl_libraries"

        CPL_CREATE_SYMBOLS
    else
        AC_MSG_RESULT([disabled])
        AC_MSG_WARN([CPL checks have been disabled! This package may not build!])
        CPL_INCLUDES=""
        CPL_LDFLAGS=""
        LIBCPLCORE=""
        LIBCPLDRS=""
        LIBCPLUI=""
        LIBCPLDFS=""
    fi

    AC_SUBST(CPL_INCLUDES)
    AC_SUBST(CPL_LDFLAGS)
    AC_SUBST(LIBCPLCORE)
    AC_SUBST(LIBCPLDRS)
    AC_SUBST(LIBCPLUI)
    AC_SUBST(LIBCPLDFS)

])
